use clap::{App, Arg};

mod day1;
mod day2;
mod day3;
mod day4;

fn main() {
    let matches = App::new("AOC 2021 Code")
        .version("0.1")
        .author("David Lenfesty <lenfesty@ualberta.ca")
        .about("My 2021 Advent of Code work.")
        .arg(
            Arg::with_name("day")
                .short("d")
                .long("day")
                .required(true)
                .takes_value(true),
        )
        .arg(
            Arg::with_name("part")
                .short("p")
                .long("part")
                .required(true)
                .takes_value(true),
        )
        .get_matches();

    // TODO bounds checking :P
    let day = matches.value_of("day").unwrap_or("1");
    let day = str::parse::<u8>(day).expect("Invalid day provided (1-25 expected)");

    let part = matches.value_of("part").unwrap_or("1");
    let part = str::parse::<u8>(part).expect("Invalid part provided (1 or 2 expected)");

    let input = read_input(day).unwrap();

    // TODO macro-ify
    match_days!(1, day1, 2, day2, 3, day3, 4, day4);
}

// TODO do the inputs ever change inside of a day?
/// Read inputs into a string
fn read_input(day: u8) -> std::io::Result<String> {
    std::fs::read_to_string(format!("inputs/day{}", day))
}

#[macro_export]
macro_rules! match_days {
    ( $($num:expr, $day:ident),* ) => {
        match day {
            $(
            $num => match part {
                1 => $day::part1(input),
                2 => $day::part2(input),
                _ => println!("Part {} invalid!", part),
            },
            )*
            _ => println!("Day {} not completed yet!", day),
        }
    };
}
